@@ -1,8 +1,9 @@
 #!/usr/bin/env python
 # coding: utf-8
 
-# Copyright (c) Saga Inc.
-# Distributed under the terms of the GPL License.
+# Copyright (c) Mito.
+# Distributed under the terms of the Modified BSD License.
+
 """
 Contains all functions that are useful for control flow. For now, this
 is just IF statements.
@@ -13,20 +14,17 @@ in more detail in docs/README.md.
 
 NOTE: This file is alphabetical order!
 """
+import pandas as pd
 import functools
 
-import pandas as pd
-from mitosheet.sheet_functions.sheet_function_utils import (
-    fill_series_with_one_index, try_extend_series_to_index)
-from mitosheet.sheet_functions.types.decorators import (
-    convert_arg_to_series_type, convert_args_to_series_type, fill_nans,
-    filter_nans, handle_sheet_function_errors)
+from mitosheet.sheet_functions.types.decorators import convert_arg_to_series_type, convert_args_to_series_type, filter_nans, fill_nans, handle_sheet_function_errors
+from mitosheet.sheet_functions.sheet_function_utils import fill_series_list_to_max, fill_series_to_length
 
 
 @handle_sheet_function_errors
 @filter_nans
-@convert_args_to_series_type('bool', 'skip', ('default', True))
-def AND(*argv: pd.Series) -> pd.Series:
+@convert_args_to_series_type('boolean_series', 'skip', ('default', True))
+def AND(*argv) -> pd.Series:
     """
     {
         "function": "AND",
@@ -48,7 +46,7 @@ def AND(*argv: pd.Series) -> pd.Series:
         ]
     }
     """
-    argv = fill_series_with_one_index(argv)
+    argv = fill_series_list_to_max(argv)
 
     return functools.reduce((lambda x, y: x & y), argv)
 
@@ -58,11 +56,11 @@ def AND(*argv: pd.Series) -> pd.Series:
 @fill_nans(0, False)
 @convert_arg_to_series_type(
     0,
-    'bool', 
+    'boolean_series', 
     'error', 
     ('default', True)
 )
-def BOOL(series: pd.Series) -> pd.Series:
+def BOOL(series) -> pd.Series:
     """
     {
         "function": "BOOL",
@@ -86,7 +84,7 @@ def BOOL(series: pd.Series) -> pd.Series:
 @filter_nans
 @convert_arg_to_series_type(
     0,
-    'bool',
+    'boolean_series',
     'error',
     'error'
 )
@@ -102,7 +100,7 @@ def BOOL(series: pd.Series) -> pd.Series:
     'error',
     'error'
 )
-def IF(condition: pd.Series, true_series: pd.Series, false_series: pd.Series) -> pd.Series:
+def IF(condition, true_series, false_series) -> pd.Series:
     """
     {
         "function": "IF",
@@ -128,19 +126,18 @@ def IF(condition: pd.Series, true_series: pd.Series, false_series: pd.Series) ->
         ]
     }
     """
-    true_series = try_extend_series_to_index(true_series, condition.index)
-    false_series = try_extend_series_to_index(false_series, condition.index)
+    true_series = fill_series_to_length(true_series, len(condition))
+    false_series = fill_series_to_length(false_series, len(condition))
 
     return pd.Series(
-        data=[true_series.loc[i] if c else false_series.loc[i] for i, c in condition.iteritems()],
-        index=condition.index
+        data=[true_series[i] if c else false_series[i] for i, c in enumerate(condition)]
     )
 
 
 @handle_sheet_function_errors
 @filter_nans
-@convert_args_to_series_type('bool', 'skip', ('default', True))
-def OR(*argv: pd.Series) -> pd.Series:
+@convert_args_to_series_type('boolean_series', 'skip', ('default', True))
+def OR(*argv) -> pd.Series:
     """
     {
         "function": "OR",
@@ -162,7 +159,7 @@ def OR(*argv: pd.Series) -> pd.Series:
     }
     """
 
-    argv = fill_series_with_one_index(argv)
+    argv = fill_series_list_to_max(argv)
     return functools.reduce((lambda x, y: x | y), argv)
 
 
@@ -173,4 +170,4 @@ CONTROL_FUNCTIONS = {
     'BOOL': BOOL,
     'IF': IF,
     'OR': OR,
-}
+}
\ No newline at end of file
