@@ -1,8 +1,9 @@
 #!/usr/bin/env python
 # coding: utf-8
 
-# Copyright (c) Saga Inc.
-# Distributed under the terms of the GPL License.
+# Copyright (c) Mito.
+# Distributed under the terms of the Modified BSD License.
+
 """
 Contains all functions that don't particularly fit in any other category
 
@@ -16,7 +17,6 @@ import pandas as pd
 import numpy as np
 import datetime
 
-from mitosheet.sheet_functions.sheet_function_utils import try_extend_series_to_index
 from mitosheet.sheet_functions.types.decorators import filter_nans, convert_arg_to_series_type, handle_sheet_function_errors
 
 
@@ -38,7 +38,6 @@ def FILLNAN(series: pd.Series, replacement: pd.Series) -> pd.Series:
     {
         "function": "FILLNAN",
         "description": "Replaces the NaN values in the series with the replacement value.",
-        "search_terms": ["fillnan", "nan", "fill nan", "missing values", "null", "null value", "fill null"],
         "examples": [
             "FILLNAN(A, 10)",
             "FILLNAN(A, 'replacement')"
@@ -55,10 +54,55 @@ def FILLNAN(series: pd.Series, replacement: pd.Series) -> pd.Series:
         ]
     }
     """
-    # Make sure the replacement series is long enough
-    replacement = try_extend_series_to_index(replacement, series.index)
+    nan_indexes = series.isna()
+    for idx in range(len(nan_indexes)):
+        if nan_indexes[idx]:
+            series[idx] = replacement[0]
 
-    return series.fillna(replacement)
+    return series
+
+
+@handle_sheet_function_errors
+@filter_nans
+@convert_arg_to_series_type(
+    0,
+    'series',
+    'error',
+    'error'
+)
+#TODO: Fix up the offset for this function, as it should be forced into a number.
+def OFFSET(series: pd.Series, offset) -> pd.Series:
+    """
+    {
+        "function": "OFFSET",
+        "description": "Shifts the given series by the given offset. Use a negative offset to reference a previous row, and a offset number to reference a later row.",
+        "examples": [
+            "OFFSET(Nums, 10)",
+            "OFFSET(call_time, -1)"
+        ],
+        "syntax": "OFFSET(series, offset)",
+        "syntax_elements": [{
+                "element": "series",
+                "description": "The series to shift up or down."
+            },
+            {
+                "element": "offset",
+                "description": "An integer amount to shift. Use a negative number to reference a previous row, and a positive number to reference a later row."
+            }
+        ]
+    }
+    """
+    length = len(series)
+    # If the shift is too big, we just return an empty series
+    if abs(offset) >= length:
+        return pd.Series([np.NaN] * length)
+    
+    if offset < 0:
+        # Otherwise, append the NaNs to the front, and chop at the correct length
+        return pd.concat([pd.Series([np.NaN] * (offset * -1)), series], ignore_index=True).head(length)
+    else:
+        remaining_series = series.tail(length - offset)
+        return pd.concat([remaining_series, pd.Series([np.NaN] * offset)], ignore_index=True).head(length)
 
 
 @handle_sheet_function_errors
@@ -73,7 +117,6 @@ def TYPE(series: pd.Series) -> pd.Series:
     {
         "function": "TYPE",
         "description": "Returns the type of each element of the passed series. Return values are 'number', 'str', 'bool', 'datetime', 'object', or 'NaN'.",
-        "search_terms": ["type", "dtype"],
         "examples": [
             "TYPE(Nums_and_Strings)",
             "IF(TYPE(Account_Numbers) != 'NaN', Account_Numbers, 0)"
@@ -110,11 +153,15 @@ def TYPE(series: pd.Series) -> pd.Series:
             return 'timedelta'
         return 'object'
 
+
     return series.apply(get_element_type).astype('str')
 
 
+
+
 # TODO: we should see if we can list these automatically!
 MISC_FUNCTIONS = {
     'FILLNAN': FILLNAN,
+    'OFFSET': OFFSET,
     'TYPE': TYPE
 }
\ No newline at end of file
